@@ -16,20 +16,33 @@
 
 import re
 import rx
-import rx.operators as ops
+import rx.operators
 import typing
 
-from datetime import datetime
 from decimal import Decimal
 from solana.publickey import PublicKey
 
 from ...accountinfo import AccountInfo
 from ...context import Context
-from ...market import Market
+from ...datetimes import utc_now
+from ...markets import Market
 from ...observables import observable_pipeline_error_reporter
-from ...oracle import Oracle, OracleProvider, OracleSource, Price, SupportedOracleFeature
+from ...oracle import (
+    Oracle,
+    OracleProvider,
+    OracleSource,
+    Price,
+    SupportedOracleFeature,
+)
 
-from .layouts import MAGIC, MAPPING, PRICE, PRODUCT, PYTH_DEVNET_MAPPING_ROOT, PYTH_MAINNET_MAPPING_ROOT
+from .layouts import (
+    MAGIC,
+    MAPPING,
+    PRICE,
+    PRODUCT,
+    PYTH_DEVNET_MAPPING_ROOT,
+    PYTH_MAINNET_MAPPING_ROOT,
+)
 
 
 # # 🥭 Pyth
@@ -59,6 +72,7 @@ from .layouts import MAGIC, MAPPING, PRICE, PRODUCT, PYTH_DEVNET_MAPPING_ROOT, P
 # Implements the `Oracle` abstract base class specialised to the Pyth Network.
 #
 
+
 class PythOracle(Oracle):
     def __init__(self, context: Context, market: Market, product_data: typing.Any):
         name = f"Pyth Oracle for {market.symbol}"
@@ -67,36 +81,53 @@ class PythOracle(Oracle):
         self.market: Market = market
         self.product_data: typing.Any = product_data
         self.address: PublicKey = product_data.address
-        features: SupportedOracleFeature = SupportedOracleFeature.MID_PRICE | SupportedOracleFeature.CONFIDENCE
+        features: SupportedOracleFeature = (
+            SupportedOracleFeature.MID_PRICE | SupportedOracleFeature.CONFIDENCE
+        )
         self.source: OracleSource = OracleSource("Pyth", name, features, market)
 
     def fetch_price(self, _: Context) -> Price:
         price_account_info = AccountInfo.load(self.context, self.product_data.px_acc)
         if price_account_info is None:
-            raise Exception(f"[{self.context.name}] Price account {self.product_data.px_acc} not found.")
+            raise Exception(
+                f"[{self.context.name}] Price account {self.product_data.px_acc} not found."
+            )
 
         if len(price_account_info.data) != PRICE.sizeof():
             raise Exception(
-                f"[{self.context.name}] Price account data has incorrect size. Expected: {PRICE.sizeof()}, got {len(price_account_info.data)}.")
+                f"[{self.context.name}] Price account data has incorrect size. Expected: {PRICE.sizeof()}, got {len(price_account_info.data)}."
+            )
 
         price_data = PRICE.parse(price_account_info.data)
         if price_data.magic != MAGIC:
-            raise Exception(f"[{self.context.name}] Price account {price_account_info.address} is not a Pyth account.")
+            raise Exception(
+                f"[{self.context.name}] Price account {price_account_info.address} is not a Pyth account."
+            )
 
         factor = Decimal(10) ** price_data.expo
         price = price_data.agg.price * factor
         confidence = price_data.agg.conf * factor
 
         # Pyth has no notion of bids, asks, or spreads so just provide the single price.
-        return Price(self.source, datetime.now(), self.market, price, price, price, confidence)
+        return Price(
+            self.source,
+            utc_now(),
+            self.market,
+            price,
+            price,
+            price,
+            confidence,
+        )
 
-    def to_streaming_observable(self, context: Context) -> rx.core.typing.Observable[Price]:
+    def to_streaming_observable(
+        self, context: Context
+    ) -> rx.core.typing.Observable[Price]:
         prices = rx.interval(1).pipe(
-            ops.observe_on(context.create_thread_pool_scheduler()),
-            ops.start_with(-1),
-            ops.map(lambda _: self.fetch_price(context)),
-            ops.catch(observable_pipeline_error_reporter),
-            ops.retry(),
+            rx.operators.observe_on(context.create_thread_pool_scheduler()),
+            rx.operators.start_with(-1),
+            rx.operators.map(lambda _: self.fetch_price(context)),
+            rx.operators.catch(observable_pipeline_error_reporter),
+            rx.operators.retry(),
         )
         return typing.cast(rx.core.typing.Observable[Price], prices)
 
@@ -109,9 +140,14 @@ class PythOracle(Oracle):
 # constructor and uses that to access the data. It ignores the context passed as a parameter to its methods.
 # This allows the context-fudging to only happen on construction.
 
+
 class PythOracleProvider(OracleProvider):
     def __init__(self, context: Context) -> None:
-        self.address: PublicKey = PYTH_MAINNET_MAPPING_ROOT if context.client.cluster_name == "mainnet" else PYTH_DEVNET_MAPPING_ROOT
+        self.address: PublicKey = (
+            PYTH_MAINNET_MAPPING_ROOT
+            if context.client.cluster_name == "mainnet"
+            else PYTH_DEVNET_MAPPING_ROOT
+        )
         self.__symbol_prefix = "Crypto."
 
         super().__init__(f"Pyth Oracle Factory [{self.address}]")
@@ -149,29 +185,39 @@ class PythOracleProvider(OracleProvider):
     def _load_pyth_mapping(self, context: Context, address: PublicKey) -> typing.Any:
         account_info = AccountInfo.load(context, address)
         if account_info is None:
-            raise Exception(f"[{context.name}] Pyth mapping account {address} not found.")
+            raise Exception(
+                f"[{context.name}] Pyth mapping account {address} not found."
+            )
 
         if len(account_info.data) != MAPPING.sizeof():
             raise Exception(
-                f"Mapping account data has incorrect size. Expected: {MAPPING.sizeof()}, got {len(account_info.data)}.")
+                f"Mapping account data has incorrect size. Expected: {MAPPING.sizeof()}, got {len(account_info.data)}."
+            )
 
         mapping: typing.Any = MAPPING.parse(account_info.data)
         mapping.address = account_info.address
         if mapping.magic != MAGIC:
-            raise Exception(f"[{context.name}] Mapping account {account_info.address} is not a Pyth account.")
+            raise Exception(
+                f"[{context.name}] Mapping account {account_info.address} is not a Pyth account."
+            )
 
         return mapping
 
-    def _fetch_all_pyth_products(self, context: Context, address: PublicKey) -> typing.Sequence[typing.Any]:
+    def _fetch_all_pyth_products(
+        self, context: Context, address: PublicKey
+    ) -> typing.Sequence[typing.Any]:
         mapping = self._load_pyth_mapping(context, address)
-        all_product_addresses = mapping.products[0:int(mapping.num)]
-        product_account_infos = AccountInfo.load_multiple(context, all_product_addresses)
+        all_product_addresses = mapping.products[0 : int(mapping.num)]
+        product_account_infos = AccountInfo.load_multiple(
+            context, all_product_addresses
+        )
         products: typing.List[typing.Any] = []
         for product_account_info in product_account_infos:
             product: typing.Any = PRODUCT.parse(product_account_info.data)
             product.address = product_account_info.address
             if product.magic != MAGIC:
                 raise Exception(
-                    f"[{context.name}] Product account {product_account_info.address} is not a Pyth account.")
+                    f"[{context.name}] Product account {product_account_info.address} is not a Pyth account."
+                )
             products += [product]
         return products
