@@ -42,7 +42,9 @@ class PerpEvent(metaclass=abc.ABCMeta):
     @abc.abstractproperty
     @property
     def accounts_to_crank(self) -> typing.Sequence[PublicKey]:
-        raise NotImplementedError("PerpEvent.accounts_to_crank is not implemented on the base type.")
+        raise NotImplementedError(
+            "PerpEvent.accounts_to_crank is not implemented on the base type."
+        )
 
     def __repr__(self) -> str:
         return f"{self}"
@@ -53,11 +55,24 @@ class PerpEvent(metaclass=abc.ABCMeta):
 # `PerpOutEvent` stores details of a perp 'fill' event.
 #
 class PerpFillEvent(PerpEvent):
-    def __init__(self, event_type: int, original_index: Decimal, timestamp: datetime, taker_side: Side,
-                 price: Decimal, quantity: Decimal, best_initial: Decimal, maker_slot: Decimal,
-                 maker_out: bool, maker: PublicKey, maker_order_id: Decimal,
-                 maker_client_order_id: Decimal, taker: PublicKey, taker_order_id: Decimal,
-                 taker_client_order_id: Decimal) -> None:
+    def __init__(
+        self,
+        event_type: int,
+        original_index: Decimal,
+        timestamp: datetime,
+        taker_side: Side,
+        price: Decimal,
+        quantity: Decimal,
+        best_initial: Decimal,
+        maker_slot: Decimal,
+        maker_out: bool,
+        maker: PublicKey,
+        maker_order_id: Decimal,
+        maker_client_order_id: Decimal,
+        taker: PublicKey,
+        taker_order_id: Decimal,
+        taker_client_order_id: Decimal,
+    ) -> None:
         super().__init__(event_type, original_index)
         self.timestamp: datetime = timestamp
         self.taker_side: Side = taker_side
@@ -85,7 +100,7 @@ class PerpFillEvent(PerpEvent):
         return [self.maker, self.taker]
 
     def __str__(self) -> str:
-        return f"""« PerpFillEvent [{self.original_index}] [{self.timestamp}] {self.taker_side} {self.quantity:,.8f} at {self.price:,.8f}
+        return f"""« PerpFillEvent [{self.original_index}] {self.taker_side} {self.quantity:,.8f} at {self.price:,.8f} [{self.timestamp}]
     Maker: {self.maker}, {self.maker_order_id} / {self.maker_client_order_id}
     Taker: {self.taker}, {self.taker_order_id} / {self.taker_client_order_id}
     Best Initial: {self.best_initial}
@@ -99,8 +114,15 @@ class PerpFillEvent(PerpEvent):
 # `PerpOutEvent` stores details of a perp 'out' event.
 #
 class PerpOutEvent(PerpEvent):
-    def __init__(self, event_type: int, original_index: Decimal, owner: PublicKey, side: Side,
-                 quantity: Decimal, slot: Decimal) -> None:
+    def __init__(
+        self,
+        event_type: int,
+        original_index: Decimal,
+        owner: PublicKey,
+        side: Side,
+        quantity: Decimal,
+        slot: Decimal,
+    ) -> None:
         super().__init__(event_type, original_index)
         self.owner: PublicKey = owner
         self.side: Side = side
@@ -120,9 +142,18 @@ class PerpOutEvent(PerpEvent):
 # `PerpLiquidateEvent` stores details of a perp 'liquidate' event.
 #
 class PerpLiquidateEvent(PerpEvent):
-    def __init__(self, event_type: int, original_index: Decimal, timestamp: datetime, seq_num: Decimal,
-                 liquidatee: PublicKey, liquidator: PublicKey, price: Decimal, quantity: Decimal,
-                 liquidation_fee: Decimal) -> None:
+    def __init__(
+        self,
+        event_type: int,
+        original_index: Decimal,
+        timestamp: datetime,
+        seq_num: Decimal,
+        liquidatee: PublicKey,
+        liquidator: PublicKey,
+        price: Decimal,
+        quantity: Decimal,
+        liquidation_fee: Decimal,
+    ) -> None:
         super().__init__(event_type, original_index)
         self.timestamp: datetime = timestamp
         self.seq_num: Decimal = seq_num
@@ -146,7 +177,9 @@ class PerpLiquidateEvent(PerpEvent):
 # the event queue data is upgraded before this code.
 #
 class PerpUnknownEvent(PerpEvent):
-    def __init__(self, event_type: int, original_index: Decimal, owner: PublicKey) -> None:
+    def __init__(
+        self, event_type: int, original_index: Decimal, owner: PublicKey
+    ) -> None:
         super().__init__(event_type, original_index)
         self.owner: PublicKey = owner
 
@@ -162,25 +195,61 @@ class PerpUnknownEvent(PerpEvent):
 #
 # `event_builder()` takes an event layout and returns a typed `PerpEvent`.
 #
-def event_builder(lot_size_converter: LotSizeConverter, event_layout: typing.Any, original_index: Decimal) -> typing.Optional[PerpEvent]:
-    if event_layout.event_type == b'\x00':
+def event_builder(
+    lot_size_converter: LotSizeConverter,
+    event_layout: typing.Any,
+    original_index: Decimal,
+) -> typing.Optional[PerpEvent]:
+    if event_layout.event_type == b"\x00":
         if event_layout.maker is None and event_layout.taker is None:
             return None
         taker_side: Side = Side.from_value(event_layout.taker_side)
-        quantity: Decimal = lot_size_converter.base_size_lots_to_number(event_layout.quantity)
+        quantity: Decimal = lot_size_converter.base_size_lots_to_number(
+            event_layout.quantity
+        )
         price: Decimal = lot_size_converter.price_lots_to_number(event_layout.price)
-        return PerpFillEvent(event_layout.event_type, original_index, event_layout.timestamp, taker_side,
-                             price, quantity, event_layout.best_initial,
-                             event_layout.maker_slot, event_layout.maker_out, event_layout.maker,
-                             event_layout.maker_order_id, event_layout.maker_client_order_id,
-                             event_layout.taker, event_layout.taker_order_id,
-                             event_layout.taker_client_order_id)
-    elif event_layout.event_type == b'\x01':
-        return PerpOutEvent(event_layout.event_type, original_index, event_layout.owner, event_layout.side, event_layout.quantity, event_layout.slot)
-    elif event_layout.event_type == b'\x02':
-        return PerpLiquidateEvent(event_layout.event_type, original_index, event_layout.timestamp, event_layout.seq_num, event_layout.liquidatee, event_layout.liquidator, event_layout.price, event_layout.quantity, event_layout.liquidation_fee)
+        return PerpFillEvent(
+            event_layout.event_type,
+            original_index,
+            event_layout.timestamp,
+            taker_side,
+            price,
+            quantity,
+            event_layout.best_initial,
+            event_layout.maker_slot,
+            event_layout.maker_out,
+            event_layout.maker,
+            event_layout.maker_order_id,
+            event_layout.maker_client_order_id,
+            event_layout.taker,
+            event_layout.taker_order_id,
+            event_layout.taker_client_order_id,
+        )
+    elif event_layout.event_type == b"\x01":
+        return PerpOutEvent(
+            event_layout.event_type,
+            original_index,
+            event_layout.owner,
+            event_layout.side,
+            event_layout.quantity,
+            event_layout.slot,
+        )
+    elif event_layout.event_type == b"\x02":
+        return PerpLiquidateEvent(
+            event_layout.event_type,
+            original_index,
+            event_layout.timestamp,
+            event_layout.seq_num,
+            event_layout.liquidatee,
+            event_layout.liquidator,
+            event_layout.price,
+            event_layout.quantity,
+            event_layout.liquidation_fee,
+        )
     else:
-        return PerpUnknownEvent(event_layout.event_type, original_index, event_layout.owner)
+        return PerpUnknownEvent(
+            event_layout.event_type, original_index, event_layout.owner
+        )
 
 
 # # 🥭 PerpEventQueue class
@@ -189,10 +258,17 @@ def event_builder(lot_size_converter: LotSizeConverter, event_layout: typing.Any
 # processed by 'consume events' and which are not.
 #
 class PerpEventQueue(AddressableAccount):
-    def __init__(self, account_info: AccountInfo, version: Version, meta_data: Metadata,
-                 head: Decimal, count: Decimal, sequence_number: Decimal,
-                 unprocessed_events: typing.Sequence[PerpEvent],
-                 processed_events: typing.Sequence[PerpEvent]) -> None:
+    def __init__(
+        self,
+        account_info: AccountInfo,
+        version: Version,
+        meta_data: Metadata,
+        head: Decimal,
+        count: Decimal,
+        sequence_number: Decimal,
+        unprocessed_events: typing.Sequence[PerpEvent],
+        processed_events: typing.Sequence[PerpEvent],
+    ) -> None:
         super().__init__(account_info)
         self.version: Version = version
 
@@ -205,7 +281,12 @@ class PerpEventQueue(AddressableAccount):
         self.processed_events: typing.Sequence[PerpEvent] = processed_events
 
     @staticmethod
-    def from_layout(layout: typing.Any, account_info: AccountInfo, version: Version, lot_size_converter: LotSizeConverter) -> "PerpEventQueue":
+    def from_layout(
+        layout: typing.Any,
+        account_info: AccountInfo,
+        version: Version,
+        lot_size_converter: LotSizeConverter,
+    ) -> "PerpEventQueue":
         meta_data: Metadata = Metadata.from_layout(layout.meta_data)
         head: Decimal = layout.head
         count: Decimal = layout.count
@@ -219,23 +300,38 @@ class PerpEventQueue(AddressableAccount):
         # Events are stored in a ringbuffer, and the oldest is overwritten when a new event arrives.
         # Make it a bit simpler to use by splitting at the insertion point and swapping the two pieces
         # around so that users don't have to do modulo arithmetic on the capacity.
-        ordered_events = events[int(head):] + events[0:int(head)]
+        ordered_events = events[int(head) :] + events[0 : int(head)]
 
         # Now chop the oldest-to-newest list of events into processed and unprocessed. The `count`
         # property holds the number of unprocessed events.
-        unprocessed_events = ordered_events[0:int(count)]
-        processed_events = ordered_events[int(count):]
+        unprocessed_events = ordered_events[0 : int(count)]
+        processed_events = ordered_events[int(count) :]
 
-        return PerpEventQueue(account_info, version, meta_data, head, count, seq_num, unprocessed_events, processed_events)
+        return PerpEventQueue(
+            account_info,
+            version,
+            meta_data,
+            head,
+            count,
+            seq_num,
+            unprocessed_events,
+            processed_events,
+        )
 
-    @ staticmethod
-    def parse(account_info: AccountInfo, lot_size_converter: LotSizeConverter) -> "PerpEventQueue":
+    @staticmethod
+    def parse(
+        account_info: AccountInfo, lot_size_converter: LotSizeConverter
+    ) -> "PerpEventQueue":
         # Data length isn't fixed so can't check we get the right value the way we normally do.
         layout = layouts.PERP_EVENT_QUEUE.parse(account_info.data)
-        return PerpEventQueue.from_layout(layout, account_info, Version.V1, lot_size_converter)
+        return PerpEventQueue.from_layout(
+            layout, account_info, Version.V1, lot_size_converter
+        )
 
-    @ staticmethod
-    def load(context: Context, address: PublicKey, lot_size_converter: LotSizeConverter) -> "PerpEventQueue":
+    @staticmethod
+    def load(
+        context: Context, address: PublicKey, lot_size_converter: LotSizeConverter
+    ) -> "PerpEventQueue":
         account_info = AccountInfo.load(context, address)
         if account_info is None:
             raise Exception(f"PerpEventQueue account not found at address '{address}'")
@@ -257,29 +353,49 @@ class PerpEventQueue(AddressableAccount):
 
         return distinct
 
-    def events_for_account(self, mango_account_address: PublicKey) -> typing.Sequence[PerpEvent]:
+    def events_for_account(
+        self, mango_account_address: PublicKey
+    ) -> typing.Sequence[PerpEvent]:
         events: typing.List[PerpEvent] = []
         for event in [*self.processed_events, *self.unprocessed_events]:
             if mango_account_address in event.accounts_to_crank:
                 events += [event]
         return events
 
-    def fills_for_account(self, mango_account_address: PublicKey) -> typing.Sequence[PerpFillEvent]:
+    def fills_for_account(
+        self, mango_account_address: PublicKey
+    ) -> typing.Sequence[PerpFillEvent]:
         fills: typing.List[PerpFillEvent] = []
         for event in self.events_for_account(mango_account_address):
             if isinstance(event, PerpFillEvent):
                 fills += [event]
         return fills
 
-    @ property
+    @property
     def capacity(self) -> int:
         return len(self.unprocessed_events) + len(self.processed_events)
 
     def __str__(self) -> str:
-        unprocessed_events = "\n        ".join([f"{event}".replace("\n", "\n        ")
-                                                for event in self.unprocessed_events if event is not None]) or "None"
-        processed_events = "\n        ".join([f"{event}".replace("\n", "\n        ")
-                                              for event in self.processed_events if event is not None]) or "None"
+        unprocessed_events = (
+            "\n        ".join(
+                [
+                    f"{event}".replace("\n", "\n        ")
+                    for event in self.unprocessed_events
+                    if event is not None
+                ]
+            )
+            or "None"
+        )
+        processed_events = (
+            "\n        ".join(
+                [
+                    f"{event}".replace("\n", "\n        ")
+                    for event in self.processed_events
+                    if event is not None
+                ]
+            )
+            or "None"
+        )
         return f"""« PerpEventQueue [{self.version}] {self.address}
     {self.meta_data}
     Head: {self.head}
@@ -312,7 +428,9 @@ class UnseenPerpEventChangesTracker:
         new_sequence_number: Decimal = event_queue.sequence_number
         if self.last_sequence_number != new_sequence_number:
             number_of_changes: Decimal = new_sequence_number - self.last_sequence_number
-            unseen = [*event_queue.processed_events, *event_queue.unprocessed_events][0 - int(number_of_changes):]
+            unseen = [*event_queue.processed_events, *event_queue.unprocessed_events][
+                0 - int(number_of_changes) :
+            ]
             self.last_sequence_number = new_sequence_number
 
         return unseen
@@ -325,14 +443,20 @@ class UnseenPerpEventChangesTracker:
 # any new fills are returned.
 #
 class UnseenAccountFillEventTracker:
-    def __init__(self, initial: PerpEventQueue, mango_account_address: PublicKey) -> None:
+    def __init__(
+        self, initial: PerpEventQueue, mango_account_address: PublicKey
+    ) -> None:
         self.mango_account_address: PublicKey = mango_account_address
         self.last_sequence_number: Decimal = initial.sequence_number
-        initial_fills: typing.Sequence[PerpFillEvent] = initial.fills_for_account(mango_account_address)
+        initial_fills: typing.Sequence[PerpFillEvent] = initial.fills_for_account(
+            mango_account_address
+        )
         self.last_key: str = initial_fills[-1].key if len(initial_fills) > 0 else ""
 
     def unseen(self, event_queue: PerpEventQueue) -> typing.Sequence[PerpEvent]:
-        fills: typing.Sequence[PerpFillEvent] = event_queue.fills_for_account(self.mango_account_address)
+        fills: typing.Sequence[PerpFillEvent] = event_queue.fills_for_account(
+            self.mango_account_address
+        )
         if len(fills) == 0:
             return []
 
@@ -349,7 +473,7 @@ class UnseenAccountFillEventTracker:
         elif last_key_position == len(fills) - 1:
             unseen = []
         else:
-            unseen = fills[last_key_position + 1:]
+            unseen = fills[last_key_position + 1 :]
             self.last_key = unseen[-1].key
 
         return unseen
